import { Archetype, Query } from "./Archetype";
import { WithRequiredKeys } from "./util/types";
/**
 * Entities in Miniplex are just plain old Javascript objects. We are assuming
 * map-like objects that use string-based properties to identify individual components.
 */
export interface IEntity {
    [key: string]: any;
}
/**
 * Miniplex uses an internal component that it will automatically add to all created
 * entities.
 */
export type MiniplexComponent<T extends IEntity> = {
    __miniplex: {
        id: number;
        world: World<T>;
        archetypes: Archetype<T>[];
    };
};
/**
 * A RegisteredEntity is an entity that has been registered with a world (and thus was blessed
 * with the __miniplex component).
 */
export type RegisteredEntity<T extends IEntity> = T & MiniplexComponent<T>;
/**
 * For situations where no entity type argument is passed to createWorld, we'll
 * default to an untyped entity type that can hold any component.
 */
export type UntypedEntity = IEntity;
/**
 * Utility type that represents an Entity that is guaranteed to have the specified
 * component(s) available.
 */
export type EntityWith<T extends IEntity, C extends keyof T> = WithRequiredKeys<T, C> & T;
/**
 * A tag is just an "empty" component. For convenience and nicer type support, we're
 * providing a Tag type and constant, both of which are simply equal to `true`.
 */
export declare const Tag = true;
export type Tag = true;
export declare class World<Entity extends IEntity = UntypedEntity> {
    /** An array holding all entities known to this world. */
    entities: RegisteredEntity<Entity>[];
    /** The ID assigned to the next entity. */
    private nextId;
    /** A list of known archetypes. */
    archetypes: Map<string, Archetype<Entity, Query<Entity>>>;
    constructor(options?: {
        entities?: Entity[];
    });
    archetype<Q extends Query<Entity>>(...query: Q): Archetype<Entity, Q>;
    private indexEntity;
    createEntity(entity: Entity): RegisteredEntity<Entity>;
    destroyEntity(entity: RegisteredEntity<Entity>): void;
    extendEntity(entity: RegisteredEntity<Entity>, components: Partial<Entity>): void;
    addComponent: <C extends keyof Entity>(entity: RegisteredEntity<Entity>, name: C, value: RegisteredEntity<Entity>[C]) => void;
    removeComponent: (entity: RegisteredEntity<Entity>, ...components: (keyof Entity)[]) => void;
    /**
     * A queue of commands to run.
     */
    private queuedCommands;
    queue: {
        createEntity: (entity: Entity) => void;
        destroyEntity: (entity: RegisteredEntity<Entity> | Entity) => void;
        extendEntity: (entity: RegisteredEntity<Entity>, components: Partial<Entity>) => void;
        addComponent: <C extends keyof Entity>(entity: RegisteredEntity<Entity>, name: C, value: RegisteredEntity<Entity>[C]) => void;
        removeComponent: (entity: RegisteredEntity<Entity>, ...names: (keyof Entity)[]) => void;
        flush: () => void;
    };
    /**
     * Clear the entire world by discarding all entities and indices.
     */
    clear(): void;
}
